# -*- coding: utf-8 -*-
from plone.formwidget.namedfile.widget import NamedFileFieldWidget  # noqa
from plone.formwidget.namedfile.widget import NamedFileWidget  # noqa
from plone.formwidget.namedfile.widget import NamedImageFieldWidget  # noqa
from plone.formwidget.namedfile.widget import NamedImageWidget  # noqa
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('plone')
